# This file is part of csvpandas
#
#    csvpandas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    csvpandas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with csvpandas.  If not, see <http://www.gnu.org/licenses/>.

"""Run stats on a dataframe
"""

import logging
import pandas

log = logging.getLogger(__name__)


def build_parser(parser):
    parser.add_argument(
        '--describe',
        action='store_true',
        help='Generate various summary statistics, excluding NaN values.')
    return parser


def action(args):
    report = []

    for col in args.csv.columns:
        for dtype in [int, float]:
            try:
                args.csv[col] = args.csv[col].astype(dtype)
                break
            except ValueError:
                continue

        if args.describe:
            report.append(args.csv[col].describe())

    report_df = pandas.concat(report, axis=1, ignore_index=True)
    report_df.to_csv(args.out)



# This file is part of csvpandas
#
#    csvpandas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    csvpandas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with csvpandas.  If not, see <http://www.gnu.org/licenses/>.
"""
This must be in its own module since it is called by setup.py and can only
load python packages included in a bare python installation.
"""

import logging
import os
import pkg_resources
import re
import subprocess


def version():
    """
    Depending on if git exists and at what version we can try these commands
    to get the git version from the git tags.  Second, if the package has been
    installed return the installed version.  Finally, if nothing else,
    return 0.0.0
    """

    version = None

    install_dir = os.path.dirname(__file__)
    git_cmds = (['git', '-C', install_dir, 'describe', '--tags'],  # >= 1.8.5
                ['git', 'describe', '--tags'])  # < 1.8.5
    devnull = open(os.devnull, 'w')

    """
    try the two git commands above ^
    git versions are organized as
    [tag]-[number of commits over tag]-[commit id]
    ex - v0.1.2-38-g6a8e5e3
    """
    for cmd in git_cmds:
        logging.debug(' '.join(cmd))
        git_re = 'v(?P<tag>[\d.]*)-?(?P<commit>[\d.]*)-?.*'
        try:
            git_ver = subprocess.check_output(cmd, stderr=devnull)
            git_search = re.search(git_re, git_ver)
            if git_search.group('commit') == '':
                version = git_search.group('tag')
            else:
                version = '{tag}.dev{commit}'.format(**git_search.groupdict())
            break
        except Exception as e:
            logging.debug('{} {}'.format(type(e), e.message))

    if version is None:
        try:
            """
            return version that was installed if available
            """
            version = pkg_resources.require("csvpandas")[0].version
        except pkg_resources.DistributionNotFound as e:
            logging.warn(e)

    return version or '0.0.0'



# This file is part of csvpandas
#
#    csvpandas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    csvpandas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with csvpandas.  If not, see <http://www.gnu.org/licenses/>.

import bz2
import gzip
import sys


def opener(mode='r'):
    """Factory for creating file objects

    Keyword Arguments:
        - mode -- A string indicating how the file is to be opened. Accepts the
            same values as the builtin open() function.
        - bufsize -- The file's desired buffer size. Accepts the same values as
            the builtin open() function.
    """

    def open_file(f):
        if f is sys.stdout or f is sys.stdin:
            return f
        elif f == '-':
            return sys.stdin if 'r' in mode else sys.stdout
        elif f.endswith('.bz2'):
            return bz2.BZ2File(f, mode)
        elif f.endswith('.gz'):
            return gzip.open(f, mode)
        else:
            return open(f, mode)

    return open_file



# This file is part of csvpandas
#
#    csvpandas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    csvpandas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with csvpandas.  If not, see <http://www.gnu.org/licenses/>.

"""Search for regex patterns
"""

import logging
import re

log = logging.getLogger(__name__)


def build_parser(parser):
    # required inputs
    parser.add_argument(
        'pattern',
        help=('search for pattern in column(s)'))

    parser.add_argument(
        '--columns',
        metavar='COLS',
        help=('Comma delimited list of column '
              'names or indices if --no-header'))
    parser.add_argument(
        '--all',
        action='store_true',
        help='pattern must exist in any column(s) [any]')
    parser.add_argument(
        '-i',
        '--ignore-case',
        action='store_true',
        help=('Ignore case distinctions in both '
              'the PATTERN and the input files.'))


def action(args):
    if args.columns:
        columns = args.columns.split(',')
    else:
        columns = args.csv.columns.tolist()

    if args.ignore_case:
        pattern = re.compile(args.pattern, re.IGNORECASE)
    else:
        pattern = re.compile(args.pattern)

    def search(string):
        return bool(re.search(pattern, string))

    df = args.csv

    if args.all:
        df = df[df[columns].apply(lambda x: x.map(search).all(), axis=1)]
    else:
        df = df[df[columns].apply(lambda x: x.map(search).any(), axis=1)]

    df.to_csv(args.out, index=False)
